==> wiiuse_v0.1_src.tar.gz: wiiuse/src/dynamics.c <==
/*
 *	wiiuse
 *
 *	Written By:
 *		Michael Laforest	< para >
 *		Email: < thepara (--AT--) g m a i l [--DOT--] com >
 *
 *	Copyright 2005
 *
 *	This file is part of wiiuse.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	$Header$
 *
 */

/**
 *	@file
 *	@brief Handles the dynamics of the wiimote.
 *
 *	The file includes functions that handle the dynamics
 *	of the wiimote.  Such dynamics include orientation and
 *	motion sensing.
 */

#include <stdio.h>

#include <math.h>

#include "definitions.h"
#include "wiiuse_internal.h"
#include "dynamics.h"


/**
 *	@brief Calculate the roll, pitch, yaw.
 *
 *	@param wm			A wiimote_t structure.
 *	@param accel		[in] Pointer to a vec3b_t structure that holds the raw acceleration data.
 *	@param orient		[out] Pointer to a orient_t structure that will hold the orientation data.
 *
 *	Given the raw acceleration data from the wiimote, calculate
 *	the orientation of the device and set it in the \a orient parameter.
 */
void calculate_orientation(struct wiimote_t* wm, struct vec3b_t* accel, struct orient_t* orient) {
	float xg, yg, zg;
	float x, y, z;

	/*
	 *	These seem to work best:
	 *
	 *	roll	- use 2*atan of the x-axis		[ ranges from -90 - 90 ]
	 *	pitch	- use 2*atan of the y-axis		[ ranges from -90 - 90 ]
	 *	yaw		- impossible to tell without IR
	 *
	 *	TODO - The tilt range sucks.  It should be -180 to 180 and work no matter
	 *			the orientation of the device.  The z-axis needs to be used somehow.
	 */

	orient->roll = 0;
	orient->pitch = 0;
	orient->yaw = 0;

	/* find out how much it has to move to be 1g */
	xg = (float)wm->cal_g.x - (float)wm->cal_zero.x;
	yg = (float)wm->cal_g.y - (float)wm->cal_zero.y;
	zg = (float)wm->cal_g.z - (float)wm->cal_zero.z;

	/* find out how much it actually moved and normalize to +/- 1g */
	x = ((float)accel->x - (float)wm->cal_zero.x) / xg;
	y = ((float)accel->y - (float)wm->cal_zero.y) / yg;
	z = ((float)accel->z - (float)wm->cal_zero.z) / zg;

	/* make sure x,y,z are between -1 and 1 for the arc functions */
	if (x < -1.0f)			x = -1.0f;
	else if (x > 1.0f)		x = 1.0f;
	if (y < -1.0f)			y = -1.0f;
	else if (y > 1.0f)		y = 1.0f;
	if (z < -1.0f)			z = -1.0f;
	else if (z > 1.0f)		z = 1.0f;

	/* if it is over 1g then it is probably accelerating and not reliable */
	if (accel->x <= wm->cal_g.x) {
		x = 2 * atanf(x);
		orient->roll = RAD_TO_DEGREE(x);
	}

	if (accel->y <= wm->cal_g.y) {
		y = 2 * atanf(y);
		orient->pitch = RAD_TO_DEGREE(y);
	}
}


/**
 *	@brief Calculate the gravity forces on each axis.
 *
 *	@param wm			A wiimote_t structure.
 *	@param accel		[in] Pointer to a vec3b_t structure that holds the raw acceleration data.
 *	@param gforce		[out] Pointer to a gforce_t structure that will hold the gravity force data.
 */
void calculate_gforce(struct wiimote_t* wm, struct vec3b_t* accel, struct gforce_t* gforce) {
	float xg, yg, zg;

	/* find out how much it has to move to be 1g */
	xg = (float)wm->cal_g.x - (float)wm->cal_zero.x;
	yg = (float)wm->cal_g.y - (float)wm->cal_zero.y;
	zg = (float)wm->cal_g.z - (float)wm->cal_zero.z;

	/* find out how much it actually moved and normalize to +/- 1g */
	gforce->x = ((float)accel->x - (float)wm->cal_zero.x) / xg;
	gforce->y = ((float)accel->y - (float)wm->cal_zero.y) / yg;
	gforce->z = ((float)accel->z - (float)wm->cal_zero.z) / zg;
}

==> wiiuse_v0.2_src.tar.gz: wiiuse/src/dynamics.c <==
/*
 *	wiiuse
 *
 *	Written By:
 *		Michael Laforest	< para >
 *		Email: < thepara (--AT--) g m a i l [--DOT--] com >
 *
 *	Copyright 2005
 *
 *	This file is part of wiiuse.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	$Header$
 *
 */

/**
 *	@file
 *	@brief Handles the dynamics of the wiimote.
 *
 *	The file includes functions that handle the dynamics
 *	of the wiimote.  Such dynamics include orientation and
 *	motion sensing.
 */

#include <stdio.h>

#include <math.h>

#include "definitions.h"
#include "wiiuse_internal.h"
#include "dynamics.h"


/**
 *	@brief Calculate the roll, pitch, yaw.
 *
 *	@param ac			An accelerometer (accel_t) structure.
 *	@param accel		[in] Pointer to a vec3b_t structure that holds the raw acceleration data.
 *	@param orient		[out] Pointer to a orient_t structure that will hold the orientation data.
 *
 *	Given the raw acceleration data from the accelerometer struct, calculate
 *	the orientation of the device and set it in the \a orient parameter.
 */
void calculate_orientation(struct accel_t* ac, struct vec3b_t* accel, struct orient_t* orient) {
	float xg, yg, zg;
	float x, y, z;

	/*
	 *	These seem to work best:
	 *
	 *	roll	- use atan(z / x)		[ ranges from -180 to 180 ]
	 *	pitch	- use atan(z / y)		[ ranges from -180 to 180 ]
	 *	yaw		- impossible to tell without IR
	 */

	orient->roll = 0;
	orient->pitch = 0;
	orient->yaw = 0;

	/* find out how much it has to move to be 1g */
	xg = (float)ac->cal_g.x - (float)ac->cal_zero.x;
	yg = (float)ac->cal_g.y - (float)ac->cal_zero.y;
	zg = (float)ac->cal_g.z - (float)ac->cal_zero.z;

	/* find out how much it actually moved and normalize to +/- 1g */
	x = ((float)accel->x - (float)ac->cal_zero.x) / xg;
	y = ((float)accel->y - (float)ac->cal_zero.y) / yg;
	z = ((float)accel->z - (float)ac->cal_zero.z) / zg;

	/* make sure x,y,z are between -1 and 1 for the arc functions */
	if (x < -1.0f)			x = -1.0f;
	else if (x > 1.0f)		x = 1.0f;
	if (y < -1.0f)			y = -1.0f;
	else if (y > 1.0f)		y = 1.0f;
	if (z < -1.0f)			z = -1.0f;
	else if (z > 1.0f)		z = 1.0f;

	/* if it is over 1g then it is probably accelerating and not reliable */
	if (accel->x <= ac->cal_g.x) {
		float s = -1;

		/* keep track of the sign but make x positive for the atan() equation */
		if (x >= 0) {
			s = 1;
			x *= -1;
		}

		x = (RAD_TO_DEGREE(atanf(z / x)) + 90);

		/* reapply the raw sign */
		if (s == -1)
			x *= s;

		orient->roll = x;
	}

	if (accel->y <= ac->cal_g.y) {
		float s = 1;

		if (y >= 0) {
			s = -1;
			y *= -1;
		}

		y = (RAD_TO_DEGREE(atanf(z / y)) + 90);

		if (s == -1)
			y *= s;

		orient->pitch = y;
	}
}


/**
 *	@brief Calculate the gravity forces on each axis.
 *
 *	@param ac			An accelerometer (accel_t) structure.
 *	@param accel		[in] Pointer to a vec3b_t structure that holds the raw acceleration data.
 *	@param gforce		[out] Pointer to a gforce_t structure that will hold the gravity force data.
 */
void calculate_gforce(struct accel_t* ac, struct vec3b_t* accel, struct gforce_t* gforce) {
	float xg, yg, zg;

	/* find out how much it has to move to be 1g */
	xg = (float)ac->cal_g.x - (float)ac->cal_zero.x;
	yg = (float)ac->cal_g.y - (float)ac->cal_zero.y;
	zg = (float)ac->cal_g.z - (float)ac->cal_zero.z;

	/* find out how much it actually moved and normalize to +/- 1g */
	gforce->x = ((float)accel->x - (float)ac->cal_zero.x) / xg;
	gforce->y = ((float)accel->y - (float)ac->cal_zero.y) / yg;
	gforce->z = ((float)accel->z - (float)ac->cal_zero.z) / zg;
}
